@@ -23,22 +23,30 @@
 #'     estimation and the estimated variance
 #'
 #' @param var.cov array of column names in dat.sub that corresponds to binary or
-#'     ordinal baseline covaraites
+#'     ordinal baseline covariates
 #'
 #' @param var.nom array of column names in dat.sub that corresponds to nominal
 #'     baseline covariates
 #'
-#' @param lst.par.pri list of prior parameters for each model:
+#' @param delta parameter for specifying the informative priors of \eqn{\sigma_g}
+#'
+#' @param prior.sig option for the informative prior on \eqn{\sigma_g}. 0: uniform prior and
+#'      1: log-normal prior
+#'
+#' @param par.pri vector of prior parameters for each model. See
+#'     \code{\link{beanz-package}} for the details of model specification.
 #'
 #' \describe{
-#'   \item{nse, fs}{\code{vtau}, \code{vrange} for (\eqn{\Delta_1, \Delta_2})}
-#'   \item{sr}{\code{vtau}, \code{vgamma}, \code{vrange}}
-#'   \item{bs, ds, eds}{\code{vtau}, \code{vw}, \code{vrange}}
-#'   \item{srs}{\code{vtau}, \code{vw}, \code{vgamma}, \code{vrange}}
+#'   \item{nse, fs}{\code{B}}
+#'   \item{sr}{\code{B}, \code{C}}
+#'   \item{bs, ds, eds}{\code{B}, \code{D}}
+#'   \item{srs}{\code{B}, \code{C}, \code{D}}
 #' }
 #'
+#' @param chains STAN options. Number of chains.
+#'
 #' @param ... options to call STAN sampling. These options include
-#'     \code{chains}, \code{iter}, \code{warmup}, \code{thin}, \code{algorithm}.
+#'     \code{iter}, \code{warmup}, \code{thin}, \code{algorithm}.
 #'     See \code{rstan::sampling} for details.
 #'
 #' @return A class \code{beanz.stan} list containing
@@ -48,10 +56,13 @@
 #'    \item{smps}{matrix of the posterior samples}
 #'    \item{get.mus}{method to return the posterior sample of the subgroup treatment effects}
 #'    \item{DIC}{DIC value}
+#'    \item{looic}{leave-one-out cross-validation information criterion}
+#'    \item{rhat}{Gelman and Rubin potential scale reduction statistic}
+#'    \item{prior.sig}{option for the informative prior on \eqn{\sigma_g}}
+#'    \item{delta}{parameter for specifying the informative priors of \eqn{\sigma_g}}
 #' }
 #'
 #' @examples
-#' \dontrun{
 #' var.cov    <- c("sodium", "lvef", "any.vasodilator.use");
 #' var.resp   <- "y";
 #' var.trt    <- "trt";
@@ -59,63 +70,111 @@
 #' resptype   <- "survival";
 #' var.estvar <- c("Estimate", "Variance");
 #'
-#' subgrp.effect <- r.get.subgrp.raw(solvd.sub,
+#' subgrp.effect <- bzGetSubgrpRaw(solvd.sub,
 #'                                   var.resp   = var.resp,
 #'                                   var.trt    = var.trt,
 #'                                   var.cov    = var.cov,
 #'                                   var.censor = var.censor,
 #'                                   resptype   = resptype);
 #'
-#' rst.nse    <- call.stan("nse", dat.sub=subgrp.effect,
+#' rst.nse    <- bzCallStan("nse", dat.sub=subgrp.effect,
 #'                          var.estvar = var.estvar, var.cov = var.cov,
-#'                          lst.par.pri = list(vtau=1000, vrange=c(0,0)),
-#'                          chains=1, iter=4000,
-#'                          warmup=2000, thin=2, seed=1000);
-#' rst.sr     <- call.stan("sr", dat.sub=subgrp.effect,
+#'                          par.pri = c(B=1000),
+#'                          chains=4, iter=600,
+#'                          warmup=200, thin=2, seed=1000);
+#'
+#' rst.sr     <- bzCallStan("sr", dat.sub=subgrp.effect,
 #'                         var.estvar=var.estvar, var.cov = var.cov,
-#'                         lst.par.pri=list(vtau=1000, vgamma=1000, vrange=c(0,0)),
-#'                         chains=1, iter=4000,
-#'                         warmup=2000, thin=2, seed=1000)};
+#'                         par.pri=c(B=1000, C=1000),
+#'                         chains=4, iter=600,
+#'                         warmup=200, thin=2, seed=1000);
 #' @export
 #'
 #'
 #'
-call.stan <- function(mdls = c("nse", "fs", "sr", "bs", "srs", "ds", "eds"),
+bzCallStan <- function(mdls = c("nse", "fs", "sr", "bs", "srs", "ds", "eds"),
                       dat.sub,
                       var.estvar,
                       var.cov,
-                      lst.par.pri,
+                      par.pri=c(B=1000,C=1000,D=1),
                       var.nom=NULL,
+                      delta = 0,
+                      prior.sig=1,
+                      chains=4,
                       ...) {
 
     mdls <- match.arg(mdls);
 
+    ##check data
+    if (!all(c(var.estvar, var.cov) %in% colnames(dat.sub)))
+        stop("Variables specified are not in the dataset.");
 
+    ##check par.pri
+    if (!all(names(par.pri) %in% c("B","C","D")))
+        stop("Prior parameters are not recognized.");
 
-    ##data manipulation
-    eval(parse(text=paste("vname <- stan.model.", mdls,
+    ##check number of chains
+    if (chains < 2)
+        stop("At least 2 Markov Chains are required in order to check convergence.")
+
+    stopifnot(all(par.pri >= 0));
+
+    ##check delta
+    stopifnot(delta >= 0);
+
+    ##check priorsig
+    stopifnot(prior.sig %in% c(0,1));
+
+    ##basic data
+    lst.basic <- list(SIZE     = nrow(dat.sub),
+                      Y        = dat.sub[,var.estvar[1]],
+                      SIGY     = sqrt(dat.sub[,var.estvar[2]]),
+                      DELTA    = delta,
+                      PRIORSIG = prior.sig);
+
+    ##model specific
+    vlist <- NULL;
+    eval(parse(text=paste("vlist <- stan.model.", mdls,
                "(dat.sub, var.estvar, var.cov, var.nom)", sep="")));
 
-    ##add prior data
-    names(lst.par.pri) <- toupper(names(lst.par.pri));
-    vname              <- c(vname, lst.par.pri);
+    ## prior data
+    names(par.pri) <- toupper(names(par.pri));
 
     ##call stan
-    stan.rst <- sampling(stanmodels[[mdls]], data=vname, ...);
+    stan.rst <- rstan::sampling(stanmodels[[mdls]],
+                         data=c(lst.basic, vlist, par.pri),
+                         pars=c("uvs", "nvs", "nomega", "nphi"),
+                         include=FALSE,
+                         chains=chains,
+                         ...);
 
-    smps <- extract(stan.rst, permuted=FALSE);
-    dic  <- get.dic(dat.sub[,var.estvar[1]], smps);
+    smps <- rstan::extract(stan.rst, permuted=FALSE);
+
+    ##diagnositics-DIC
+    dic   <- get.dic(dat.sub[,var.estvar[1]], smps);
+    ##diagnositics-LOOIC
+    looic <- loo::loo(loo::extract_log_lik(stan.rst), cores = 1);
+    ##rhat
+    rhat  <- rstan::summary(stan.rst)$summary[,"Rhat"];
 
     ##return
-    rst <- list(mdl      = get.mdl.name(mdls),
-                stan.rst = stan.rst,
-                smps     = smps,
-                dic      = dic,
+    rst <- list(mdl       = get.mdl.name(mdls),
+                stan.rst  = stan.rst,
+                smps      = smps,
+                dic       = dic,
+                looic     = looic,
+                prior.sig = prior.sig,
+                delta     = delta,
+                rhat      = rhat,
                 get.mus=function() {
-
         n.mu <- stan.rst@par_dims$mu;
-        mj   <- paste("mu[", 1:n.mu, "]", sep="");
-        mus  <- smps[,1,mj];
+        mj    <- paste("mu[", 1:n.mu, "]", sep="");
+        s.mus <- smps[,,mj];
+
+        mus <- NULL;
+        for (k in 1:dim(s.mus)[2]) {
+            mus <- rbind(mus, s.mus[,k,]);
+        }
         colnames(mus) <- paste("Subgroup", 1:n.mu);
         mus
     })
@@ -126,31 +185,15 @@ call.stan <- function(mdls = c("nse", "fs", "sr", "bs", "srs", "ds", "eds"),
 
 ##no subgroup effect model
 stan.model.nse <- function(dat.sub, var.estvar, var.cov, var.nom) {
-    ##data manipulation
-    SIZE   <- nrow(dat.sub);
-    Y      <- dat.sub[,var.estvar[1]];
-    VY     <- dat.sub[,var.estvar[2]];
-    vname  <- make.list(environment());
-    vname
+    NULL;
 }
-
 ##full stratification
-stan.model.fs <- function(dat.sub, var.estvar, var.cov, var.nom) {
-    ##data manipulation
-    SIZE  <- nrow(dat.sub);
-    Y     <- dat.sub[,var.estvar[1]];
-    VY    <- dat.sub[,var.estvar[2]];
-    vname <- make.list(environment());
-    vname
-}
+stan.model.fs <- stan.model.nse;
+##basic shrinkage
+stan.model.bs <- stan.model.fs;
 
 ##simple regression
 stan.model.sr <- function(dat.sub, var.estvar, var.cov, var.nom) {
-    ##data manipulation
-    Y      <- dat.sub[,var.estvar[1]];
-    SIZE   <- nrow(dat.sub);
-    VY     <- dat.sub[,var.estvar[2]];
-
     ##design matrix
     dx     <- dat.sub[,var.cov, drop=FALSE];
     dx     <- df.convert(dx, var.nom);
@@ -163,8 +206,6 @@ stan.model.sr <- function(dat.sub, var.estvar, var.cov, var.nom) {
     vname
 }
 
-##basic shrinkage
-stan.model.bs <- stan.model.fs;
 ##simple regression + shrinkage
 stan.model.srs <- stan.model.sr;
 ##Dixon and Simon
@@ -172,11 +213,6 @@ stan.model.ds <- stan.model.sr;
 
 ##extended dixon and simon
 stan.model.eds <- function(dat.sub, var.estvar, var.cov, var.nom) {
-    ##data manipulation
-    SIZE   <- nrow(dat.sub);
-    Y      <- dat.sub[,var.estvar[1]];
-    VY     <- dat.sub[,var.estvar[2]];
-
     ##design matrix
     dx     <- dat.sub[,var.cov, drop=FALSE];
     dx     <- df.convert(dx, var.nom);
@@ -224,7 +260,7 @@ make.list <- function(par.env) {
 ##                             DIC
 ##-----------------------------------------------------------------------------------
 get.log.ll <- function(y, mu, vs) {
-    rst <- dnorm(y, mu, sqrt(vs), log=TRUE);
+    rst <- dnorm(y, mu, vs, log=TRUE);
     sum(rst);
 }
 
@@ -232,20 +268,16 @@ get.dic <- function(y, stan.smps) {
     ny    <- length(y);
     l.mus <- paste("mu[", 1:ny, "]", sep="");
     l.vs  <- paste("vs[", 1:ny, "]", sep="");
+    l.ll  <- paste("log_lik[", 1:ny, "]", sep="");
 
-    mus   <- stan.smps[, 1, l.mus];
-    vs    <- stan.smps[, 1, l.vs];
-
-    mean.mu <- apply(mus, 2, mean);
-    mean.vs <- 1/(apply(1/vs, 2, mean));
+    mus     <- stan.smps[, , l.mus];
+    vs      <- stan.smps[, , l.vs];
+    ll      <- stan.smps[, , l.ll];
+    mean.mu <- apply(mus, 3, mean);
+    mean.vs <- 1/(apply(1/vs, 3, mean));
     d.theta.bar <- -2*get.log.ll(y, mean.mu, mean.vs);
-
-    all.d   <- apply(cbind(mus, vs), 1,
-                     function(x) {
-                         rst <- -2*get.log.ll(y, x[1:ny], x[(ny+1):(2*ny)]);
-                         rst
-                     });
-    bar.d   <- mean(all.d);
+    all.d       <- -2*apply(ll, c(1,2), sum);
+    bar.d       <- mean(all.d);
 
     ##dic
     DIC <- 2*bar.d - d.theta.bar;
